/******************************************************************************
 *    AnimTextureService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IAnimTextureSrv>::Name = "AnimTextureService";

	const char * 
	ScriptService<IAnimTextureSrv>::DocStrings[] = {
		"AnimTextureService()\n\t"
			"Allows for textures on brush faces to be swapped.",
		"ChangeTexture(obj,from_fam,from_tex,to_fam,to_tex) -> Integer\n\t"
			"Swaps textures surrounding obj. The fam strings may be None,\n\t"
			"in which case the tex strings must be the full path to the\n\t"
			"textures. Otherwise, the fam strings are family paths, and\n\t"
			"tex are the texture names.\n\t"
			"Returns 0 on success.",
	};

	const PyMethodDef 
	ScriptService<IAnimTextureSrv>::Methods[] = {
		{	"ChangeTexture",
			reinterpret_cast<PyCFunction>(ChangeTexture),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IAnimTextureSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IAnimTextureSrv>::Name),
		sizeof(ScriptServiceObject<IAnimTextureSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IAnimTextureSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IAnimTextureSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IAnimTextureSrv> * 
	ScriptService<IAnimTextureSrv>::NewObject(PyObject *)
	{
		IAnimTextureSrv *iface = static_cast<IAnimTextureSrv*>(g_pScriptManager->GetService(IID_IAnimTextureScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IAnimTextureSrv> *obj = PyObject_New(ScriptServiceObject<IAnimTextureSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IAnimTextureSrv>::ChangeTexture(ScriptServiceObject<IAnimTextureSrv> *self, PyObject *args)
	{
		object obj;
		const char *from1, *from2, *to1, *to2;
		from1 = to1 = NULL;
		if (!PyArg_ParseTuple(args, "O&zszs:ChangeTexture", cvtObject, &obj, &from1, &from2, &to1, &to2))
		{
			return NULL;
		}
		long ret = self->m_pIFace->ChangeTexture(obj,from1,from2,to1,to2);
		return PyInt_FromLong(ret);
	}


} // namespace PyNumbra


