/******************************************************************************
 *    LockService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<ILockSrv>::Name = "LockService";

	const char * 
	ScriptService<ILockSrv>::DocStrings[] = {
		"LockService()\n\t"
			"Allows you to determine whether an object is locked or not.",
		"IsLocked(obj) -> Boolean\n\t"
			"Checks Lock links as well as the property.",
	};

	const PyMethodDef 
	ScriptService<ILockSrv>::Methods[] = {
		{	"IsLocked",
			reinterpret_cast<PyCFunction>(IsLocked),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<ILockSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<ILockSrv>::Name),
		sizeof(ScriptServiceObject<ILockSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<ILockSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<ILockSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<ILockSrv> * 
	ScriptService<ILockSrv>::NewObject(PyObject *)
	{
		ILockSrv *iface = static_cast<ILockSrv*>(g_pScriptManager->GetService(IID_ILockedScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<ILockSrv> *obj = PyObject_New(ScriptServiceObject<ILockSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<ILockSrv>::IsLocked(ScriptServiceObject<ILockSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:IsLocked", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsLocked(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}


} // namespace PyNumbra


