/******************************************************************************
 *    PickLockService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IPickLockSrv>::Name = "PickLockService";

	const char * 
	ScriptService<IPickLockSrv>::DocStrings[] = {
		"PickLockService()\n\t"
			"Allows you to the lockpicks and the lock that is being picked.",
		"Ready(frobber,pick) -> Integer",
		"UnReady(frobber,pick) -> Integer",
		"StartPicking(frobber,pick,obj) -> Integer",
		"FinishPicking(pick) -> Integer",
		"CheckPick(pick,obj,i) -> Integer",
		"DirectMotion(i) -> Integer",
	};

	const PyMethodDef 
	ScriptService<IPickLockSrv>::Methods[] = {
		{	"Ready",
			reinterpret_cast<PyCFunction>(Ready),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"UnReady",
			reinterpret_cast<PyCFunction>(UnReady),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"StartPicking",
			reinterpret_cast<PyCFunction>(StartPicking),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"FinishPicking",
			reinterpret_cast<PyCFunction>(FinishPicking),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"CheckPick",
			reinterpret_cast<PyCFunction>(CheckPick),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"DirectMotion",
			reinterpret_cast<PyCFunction>(DirectMotion),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IPickLockSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IPickLockSrv>::Name),
		sizeof(ScriptServiceObject<IPickLockSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IPickLockSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IPickLockSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IPickLockSrv> * 
	ScriptService<IPickLockSrv>::NewObject(PyObject *)
	{
		IPickLockSrv *iface = static_cast<IPickLockSrv*>(g_pScriptManager->GetService(IID_IPickLockScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IPickLockSrv> *obj = PyObject_New(ScriptServiceObject<IPickLockSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IPickLockSrv>::Ready(ScriptServiceObject<IPickLockSrv> *self, PyObject *args)
	{
		object frobber, pick;
		if (!PyArg_ParseTuple(args, "O&O&:Ready", cvtObject, &frobber, cvtObject, &pick))
		{
			return NULL;
		}
		int ret = self->m_pIFace->Ready(frobber,pick);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPickLockSrv>::UnReady(ScriptServiceObject<IPickLockSrv> *self, PyObject *args)
	{
		object frobber, pick;
		if (!PyArg_ParseTuple(args, "O&O&:UnReady", cvtObject, &frobber, cvtObject, &pick))
		{
			return NULL;
		}
		int ret = self->m_pIFace->UnReady(frobber,pick);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPickLockSrv>::StartPicking(ScriptServiceObject<IPickLockSrv> *self, PyObject *args)
	{
		object frobber, pick, obj;
		if (!PyArg_ParseTuple(args, "O&O&O&:StartPicking", cvtObject, &frobber, cvtObject, &pick, cvtObject, &obj))
		{
			return NULL;
		}
		int ret = self->m_pIFace->StartPicking(frobber,pick,obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPickLockSrv>::FinishPicking(ScriptServiceObject<IPickLockSrv> *self, PyObject *args)
	{
		object pick;
		if (!PyArg_ParseTuple(args, "O&:FinishPicking", cvtObject, &pick))
		{
			return NULL;
		}
		int ret = self->m_pIFace->FinishPicking(pick);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPickLockSrv>::CheckPick(ScriptServiceObject<IPickLockSrv> *self, PyObject *args)
	{
		int i;
		object pick, obj;
		if (!PyArg_ParseTuple(args, "O&O&i:CheckPick", cvtObject, &pick, cvtObject, &obj, &i))
		{
			return NULL;
		}
		int ret = self->m_pIFace->CheckPick(pick,obj,i);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPickLockSrv>::DirectMotion(ScriptServiceObject<IPickLockSrv> *self, PyObject *args)
	{
		int i;
		if (!PyArg_ParseTuple(args, "i:DirectMotion", &i))
		{
			return NULL;
		}
		int ret = self->m_pIFace->DirectMotion(i);
		return PyInt_FromLong(ret);
	}


} // namespace PyNumbra


