/******************************************************************************
 *    PuppetService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IPuppetSrv>::Name = "PuppetService";

	const char * 
	ScriptService<IPuppetSrv>::DocStrings[] = {
		"PuppetService()\n\t"
			"Pulls the strings.",
		"PlayMotion(obj,name) -> Boolean\n\t"
			"Plays a motion schema on obj.",
	};

	PyMethodDef 
	const ScriptService<IPuppetSrv>::Methods[] = {
		{	"PlayMotion",
			reinterpret_cast<PyCFunction>(PlayMotion),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IPuppetSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IPuppetSrv>::Name),
		sizeof(ScriptServiceObject<IPuppetSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IPuppetSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IPuppetSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IPuppetSrv> * 
	ScriptService<IPuppetSrv>::NewObject(PyObject *)
	{
		IPuppetSrv *iface = static_cast<IPuppetSrv*>(g_pScriptManager->GetService(IID_IPuppetScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IPuppetSrv> *obj = PyObject_New(ScriptServiceObject<IPuppetSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IPuppetSrv>::PlayMotion(ScriptServiceObject<IPuppetSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		if (!PyArg_ParseTuple(args, "O&s:PlayMotion", cvtObject, &obj, &name))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->PlayMotion(ret,obj,name);
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}


} // namespace PyNumbra


