/******************************************************************************
 *    ScriptInterpreter.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef PYNUMBRA_SCRIPTINTERPRETER_H
#define PYNUMBRA_SCRIPTINTERPRETER_H

#include <Python.h>
#include <map>
#include <string>
#include <functional>
#include <cstring>
#include <windows.h>
#include <lg/types.h>

#include "PyNumbra.h"

namespace PyNumbra
{

	class CScriptInterpreter : public IScriptInterpreter
	{
		struct strless_nocase : public std::binary_function<const char*,const char*,bool>
		{
			bool operator()(const char* __x, const char* __y) const
				{ return ::stricmp(__x,__y) < 0; }
		};

		unsigned long m_iRefCount;

		bool m_bInitialized;

		char *m_pPythonHome;
		char * *m_pScriptNames;
		char * *m_pScriptPath;
		unsigned int m_iScriptPathSize;

		typedef std::map<const char*,PyObject*,strless_nocase> _classmap_type;
		typedef std::map<const char*,const char*,strless_nocase> _filemap_type;

		_classmap_type m_Classes;
		_filemap_type m_ScriptFiles;

		unsigned long m_iLoadCount;
		bool m_bEvalLock;

	public:
		virtual ~CScriptInterpreter();
		CScriptInterpreter();

		virtual unsigned long __stdcall AddRef();
		virtual unsigned long __stdcall Release();
		virtual HRESULT __stdcall QueryInterface(REFIID, void**);

		virtual void __stdcall SetPythonHome(const char *pszPath);
		virtual int __stdcall SetScriptPath(const char *pszPath);
		virtual const char * const * __stdcall GetScriptNames(void);
		virtual void __stdcall InvalidateScriptCache(void);
		virtual PyObject * __stdcall LoadScript(const char *pszName);
		virtual void __stdcall UnloadScript(const char *pszName, PyObject *pClass);
		virtual PyObject * __stdcall BeginScript(const char *pszName, PyObject *pClass, int iObjId);
		virtual void __stdcall EndScript(PyObject *pScript, PyObject *pClass, int iObjId);
		virtual int __stdcall ReceiveMessage(PyObject *pScript, const sScrMsg *pMsg, sMultiParm *pReply);
		virtual int __stdcall Eval(const char *pszCmd);

	protected:
		int Initialize(void);
		void Finalize(void);
		unsigned long Load(void);
		void Unload(void);

		static const char * g_pNull;

	};

} // namespace PyNumbra

#endif
