/******************************************************************************
 *    ScriptMessages.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include <lg/scrmsgs.h>

namespace PyNumbra
{
	class ScriptMessages
	{
	public:
		static sScrMsg * MessageFromObject(PyObject *dict);
		static PyObject * MessageToObject(const sScrMsg *msg);

	private:
		static sScrMsg * _NewMsg(const char *pszName, PyObject *dict);
		static void _NewObj(const sScrMsg *msg, PyObject *msgDict);
		
		struct _MsgTypeStruct
		{
			const char * Name;
			sScrMsg * (*NewMsg)(PyObject *msgDict);
			void (*NewObj)(const sScrMsg *msg, PyObject *msgDict);

			inline bool operator== (const char *pszKey) const;
		};

		// Keep this list sorted. We do a binary search.
		static const _MsgTypeStruct _MsgTypes[];
		static const _MsgTypeStruct * _MsgLast;
		static const int NUM_MSGTYPES;

		// TurnOn and TurnOff are used so frequently, even though
		// they don't have any extra data.
		static sScrMsg * NewGenericMsg(PyObject *msgDict);
		static void NewGenericObj(const sScrMsg *msg, PyObject *msgDict);

		// This message has a return value in the struct, 
		// which makes it unfeasable to call this way.
		//static sScrMsg * NewScriptPtrQueryMsg(PyObject *msgDict);
		//static void NewScriptPtrQueryObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewSimMsg(PyObject *msgDict);
		static void NewSimObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewAIModeChangeMsg(PyObject *msgDict);
		static void NewAIModeChangeObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewAlertnessMsg(PyObject *msgDict);
		static void NewAlertnessObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewHighAlertMsg(PyObject *msgDict);
		static void NewHighAlertObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewObjActResultMsg(PyObject *msgDict);
		static void NewObjActResultObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewPatrolPointMsg(PyObject *msgDict);
		static void NewPatrolPointObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewSignalAIMsg(PyObject *msgDict);
		static void NewSignalAIObj(const sScrMsg *msg, PyObject *msgDict);
		// "StartAttack" "EndAttack" "StartWindup"
		static sScrMsg * NewAttackMsg(PyObject *msgDict);
		static void NewAttackObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewCombineMsg(PyObject *msgDict);
		static void NewCombineObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewContainedMsg(PyObject *msgDict);
		static void NewContainedObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewContainerMsg(PyObject *msgDict);
		static void NewContainerObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewDamageMsg(PyObject *msgDict);
		static void NewDamageObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewDifficultyMsg(PyObject *msgDict);
		static void NewDifficultyObj(const sScrMsg *msg, PyObject *msgDict);
		// "DoorClosed" "DoorOpen" "DoorClosing" "DoorOpening" "DoorHalt"
		static sScrMsg * NewDoorMsg(PyObject *msgDict);
		static void NewDoorObj(const sScrMsg *msg, PyObject *msgDict);
		// "FrobWorldBegin" "FrobWorldEnd" "FrobInvBegin" "FrobInvEnd" "FrobToolBegin" "FrobToolEnd"
		// "WorldFocus" "WorldDeFocus" "WorldSelect" "WorldDeSelect" "InvFocus" "InvDeFocus"
		// "InvSelect" "InvDeSelect"
		static sScrMsg * NewFrobMsg(PyObject *msgDict);
		static void NewFrobObj(const sScrMsg *msg, PyObject *msgDict);
		// "MotionStart" "MotionEnd" "MotionFlagReached"
		static sScrMsg * NewMotionMsg(PyObject *msgDict);
		static void NewMotionObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewPickStateChangeMsg(PyObject *msgDict);
		static void NewPickStateChangeObj(const sScrMsg *msg, PyObject *msgDict);
		// "PressurePlateActivating" "PressurePlateDeactivating" "PressurePlateActive" "PressurePlateInactive"
		static sScrMsg * NewPressurePlateMsg(PyObject *msgDict);
		static void NewPressurePlateObj(const sScrMsg *msg, PyObject *msgDict);
		// "PhysExit" "PhysEnter" "PhysContactDestroy" "PhysContactCreate" "PhysCollision" 
		// "PhysMadeNonPhysical" "PhysMadePhysical" "PhysWokeUp" "PhysFellAsleep"
		static sScrMsg * NewPhysMsg(PyObject *msgDict);
		static void NewPhysObj(const sScrMsg *msg, PyObject *msgDict);
		// I don't think we can use this one either
		//static sScrMsg * NewReportMessageMsg(PyObject *msgDict);
		//static void NewReportMessageObj(const sScrMsg *msg, PyObject *msgDict);
		// "PlayerRoomEnter" "PlayerRoomExit" "RemotePlayerRoomEnter" "RemotePlayerRoomExit"
		// "CreatureRoomEnter" "CreatureRoomExit" "ObjectRoomEnter" "ObjectRoomExit" "ObjRoomTransit"
		static sScrMsg * NewRoomMsg(PyObject *msgDict);
		static void NewRoomObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewSlainMsg(PyObject *msgDict);
		static void NewSlainObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewSchemaDoneMsg(PyObject *msgDict);
		static void NewSchemaDoneObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewSoundDoneMsg(PyObject *msgDict);
		static void NewSoundDoneObj(const sScrMsg *msg, PyObject *msgDict);
		// This is treated specially: Any message that ends with "Stimulus"
		static sScrMsg * NewStimMsg(PyObject *msgDict);
		static void NewStimObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewTimerMsg(PyObject *msgDict);
		static void NewTimerObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewTweqCompleteMsg(PyObject *msgDict);
		static void NewTweqCompleteObj(const sScrMsg *msg, PyObject *msgDict);
		// "Waypoint" "WaypointReached"
		static sScrMsg * NewWaypointMsg(PyObject *msgDict);
		static void NewWaypointObj(const sScrMsg *msg, PyObject *msgDict);
		// "MovingTerrainWaypoint" "Call"
		static sScrMsg * NewMovingTerrainMsg(PyObject *msgDict);
		static void NewMovingTerrainObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewQuestChangeMsg(PyObject *msgDict);
		static void NewQuestChangeObj(const sScrMsg *msg, PyObject *msgDict);
		static sScrMsg * NewMediumTransMsg(PyObject *msgDict);
		static void NewMediumTransObj(const sScrMsg *msg, PyObject *msgDict);
		// "YorNDone"
		static sScrMsg * NewYorNMsg(PyObject *msgDict);
		static void NewYorNObj(const sScrMsg *msg, PyObject *msgDict);
		// "KeypadDone"
		static sScrMsg * NewKeypadMsg(PyObject *msgDict);
		static void NewKeypadObj(const sScrMsg *msg, PyObject *msgDict);

	};

} // namespace PyNumbra

