/******************************************************************************
 *    ScriptModule.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <new>
#include <cstring>
#include <cctype>
#include <sys/stat.h>
#include <windows.h>
#include <objbase.h>

#include <lg/scrmanagers.h>
#include <lg/scrservices.h>

#include <hacks/dromedhack.h>
#include <hacks/darkprocs.h>
#include <hacks/allhacks.h>

#include "ScriptModule.h"
#include "PyScript.h"

static config_get_raw pfnConfigGet = NULL;
static int ConfigGet(const char *name, char *buffer, int buffer_size);
mprintf g_pfnMPrintf = NULL;
static HMODULE g_hPyNumbraModule = NULL;
PyNumbra::IScriptInterpreter *g_pScriptInterpreter = NULL;
IMalloc *g_pMalloc = NULL;
IScriptMan *g_pScriptManager = NULL;

typedef BOOL (WINAPI *fnSetDllDirectoryA)(LPCSTR);

cScriptModule::~cScriptModule()
{
	if (sm_ScriptsArray)
		delete[] sm_ScriptsArray;
	if (m_pszName)
		delete[] m_pszName;
}

cScriptModule::cScriptModule()
	    : m_iRef(1), m_pszName(NULL) //, m_iCurIndex(0)
{
}

cScriptModule::cScriptModule(const char* pszName)
	    : m_iRef(1) //, m_iCurIndex(0)
{
	SetName(pszName);
}

HRESULT cScriptModule::QueryInterface(REFIID riid, void** ppout)
{
	if (riid == IID_IUnknown)
		*ppout = this;
	else if (riid == IID_IScriptModule)
		*ppout = static_cast<IScriptModule*>(this);
	else
		return E_NOINTERFACE;
	reinterpret_cast<IUnknown*>(*ppout)->AddRef();
	return S_OK;
}

ULONG cScriptModule::AddRef(void) 
{
	return m_iRef++;
}

ULONG cScriptModule::Release(void)
{
	// global COM objects?  First time I've ever heard of THAT
	if (m_iRef)
		m_iRef--;
	return m_iRef;
}
	
const char* cScriptModule::GetName(void)
{
	return m_pszName;
}

void cScriptModule::SetName(const char* pszName)
{
	m_pszName = new(std::nothrow) char[strlen(pszName)];
	strcpy(m_pszName, pszName);
}

const sScrClassDesc* cScriptModule::GetFirstClass(tScrIter* pIterParam)
{
	const sScrClassDesc *pRet;
	//m_iCurIndex = 0;
	*reinterpret_cast<unsigned int*>(pIterParam) = 0;
	pRet = GetScript(0);
	return pRet;
}

const sScrClassDesc* cScriptModule::GetNextClass(tScrIter* pIterParam) 
{
	const sScrClassDesc *pRet;
	register unsigned int index = *reinterpret_cast<unsigned int*>(pIterParam);
	//m_iCurIndex++;
	pRet = GetScript(++index);
	*reinterpret_cast<unsigned int*>(pIterParam) = index;

	return pRet;
}

void cScriptModule::EndClassIter(tScrIter*)
{
	// Nothing to do here
}

const sScrClassDesc* cScriptModule::GetScript(unsigned int i)
{
	if (i < sm_ScriptsArraySize)
		return &sm_ScriptsArray[i];
	else
		return NULL;
}

void cScriptModule::InitPath(void)
{
	struct stat sInfo;
	char chPath[1024];
	if (ConfigGet("python_script_path", chPath, 1024))
	{
		g_pScriptInterpreter->SetScriptPath(chPath);
		return;
	}
	if (0 != stat("python/scripts", &sInfo) || !S_ISDIR(sInfo.st_mode))
		return;
	chPath[0] = '.'; chPath[1] = '\0';
	GetCurrentDirectoryA(1016,chPath);
	strcat(chPath, "\\Python");
	g_pScriptInterpreter->SetScriptPath(chPath);
}

void cScriptModule::BuildScripts(void)
{
	const char * const * pNames;
	pNames = g_pScriptInterpreter->GetScriptNames();
	if (!pNames || !*pNames)
		return;
	unsigned int n = 0;
	const char * const * p;
	for (p = pNames; *p; ++p)
		++n;

	sScrClassDesc *pScripts = new sScrClassDesc[n];
	n = 0;
	for (p = pNames; *p; ++p)
	{
		pScripts[n].pszModule = m_pszName;
		pScripts[n].pszBaseClass = "PyScript";
		pScripts[n].pszClass = const_cast<char*>(*p);
		pScripts[n].pfnFactory = reinterpret_cast<ScriptFactoryProc>(&PyScriptFactory);
		++n;
	}
	if (sm_ScriptsArray)
		delete[] sm_ScriptsArray;
	sm_ScriptsArray = pScripts;
	sm_ScriptsArraySize = n;
}

PyNumbra::IScriptInterpreter * cScriptModule::InitPyNumbra(void)
{
	UINT prevmode = SetErrorMode(SEM_NOOPENFILEERRORBOX);
	char chHome[260];
	chHome[0] = '\0';
	if (!ConfigGet("python_home",chHome,246))
	{
		/* Just work with the system-wide python home by default */
		g_hPyNumbraModule = LoadLibraryA("PyNumbra.dll");
	}
	else
	{
		HMODULE hKernel32 = GetModuleHandleA("KERNEL32.dll");
		fnSetDllDirectoryA pSetDllDirectory = 
			reinterpret_cast<fnSetDllDirectoryA>(GetProcAddress(hKernel32, "SetDllDirectoryA"));
		if (pSetDllDirectory != NULL)
		{
			// SetDllDirectory is only available in WinXP,
			// and it is only needed in WinXP. So if it exists, we use it.
			pSetDllDirectory(chHome);
			g_hPyNumbraModule = LoadLibraryA("PyNumbra.dll");
			pSetDllDirectory(NULL);
		}
		else
		{
			// Granted, it would have been a hella easier if
			// SetDllDirectory were always available.
			char chPath[260];
			GetCurrentDirectoryA(246,chPath);
			char *dirsep = chPath + strlen(chPath);
			strcat(chPath, "\\PyNumbra.dll");
			SetCurrentDirectoryA(chHome);
			g_hPyNumbraModule = LoadLibraryA(chPath);
			*dirsep = '\0';
			SetCurrentDirectoryA(chPath);
		}
	}
	SetErrorMode(prevmode);
	if (!g_hPyNumbraModule)
		return NULL;
	
	PyNumbraInitProc pfnPyNumbraInit = 
		reinterpret_cast<PyNumbraInitProc>(GetProcAddress(g_hPyNumbraModule, "PyNumbraInit"));
	if (!pfnPyNumbraInit)
	{
		//FreeLibrary(g_hPyNumbraModule);
		//g_hPyNumbraModule = NULL;
		return NULL;
	}
	PyNumbra::IScriptInterpreter *pInterpreter = 
		pfnPyNumbraInit((chHome[0] != '\0') ? chHome : NULL, g_pScriptManager, g_pMalloc, g_pfnMPrintf);
	return pInterpreter;
}

int ConfigGet(const char *name, char *buffer, int buffer_size)
{
	if (pfnConfigGet)
	{
		return (pfnConfigGet(name,buffer,buffer_size)) ? 1 : 0;
	}
	// The hard way.
	char chLine[1024];
	int namelen = strlen(name);
	HANDLE cfgfile;
	DWORD dwBytes;
#if (_DARKGAME == 2)
	cfgfile = CreateFileA("darkinst.cfg", GENERIC_READ, FILE_SHARE_READ, NULL, 
					OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (cfgfile != INVALID_HANDLE_VALUE)
	{
		while (ReadFile(cfgfile, chLine, sizeof(chLine), &dwBytes, NULL))
		{
			if (dwBytes == 0)
				break;
			if (!strnicmp(chLine,name,namelen) && isspace(chLine[namelen]))
			{
				if (!buffer)
					return 1;
				char* l = &chLine[namelen+1];
				while (isspace(*l)) ++l;
				char* k = l + strlen(l) - 1;
				while (isspace(*k)) --k;
				++k;
				*k = '\0';
				strncpy(buffer, l, buffer_size);
				return 1;
			}
		}
		CloseHandle(cfgfile);
	}
#endif
	cfgfile = CreateFileA("install.cfg", GENERIC_READ, FILE_SHARE_READ, NULL, 
					OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (cfgfile != INVALID_HANDLE_VALUE)
	{
		while (ReadFile(cfgfile, chLine, sizeof(chLine), &dwBytes, NULL))
		{
			if (dwBytes == 0)
				break;
			if (!strnicmp(chLine,name,namelen) && isspace(chLine[namelen]))
			{
				if (!buffer)
					return 1;
				char* l = &chLine[namelen+1];
				while (isspace(*l)) ++l;
				char* k = l + strlen(l) - 1;
				while (isspace(*k)) --k;
				++k;
				*k = '\0';
				strncpy(buffer, l, buffer_size);
				return 1;
			}
		}
		CloseHandle(cfgfile);
	}
	return 0;
}


sScrClassDesc *cScriptModule::sm_ScriptsArray = NULL;
unsigned int cScriptModule::sm_ScriptsArraySize = 0;

cScriptModule  g_ScriptModule;

extern "C" __declspec(dllexport) __stdcall 
int ScriptModuleInit (char* pszName, IScriptMan* pScriptMan, void (*pfnMPrintf)(), IMalloc* pMalloc, IScriptModule** pOutInterface)
{
	*pOutInterface = NULL;

	g_pScriptManager = pScriptMan;
	g_pMalloc = pMalloc;
	if (!pMalloc)
		pScriptMan->QueryInterface(IID_IMalloc, reinterpret_cast<void**>(g_pMalloc));
	g_pfnMPrintf = reinterpret_cast<mprintf>(pfnMPrintf);

	if (!g_pScriptManager || !g_pMalloc)
		return 0;

	g_pScriptInterpreter = g_ScriptModule.InitPyNumbra();
	if (!g_pScriptInterpreter)
	{
		g_pfnMPrintf("!!! Could not load PyNumbra.dll.\n\n");
		FreeLibrary(g_hPyNumbraModule);
		g_hPyNumbraModule = NULL;
		return 0;
	}
	try
	{
		g_ScriptModule.InitPath();
		g_ScriptModule.BuildScripts();
	}
	catch (...)
	{
		g_pfnMPrintf("!!! Could not enumerate Python scripts.\n\n");
		return 0;
	}

	//g_pScriptModule = new cScriptModule(pszName);
	g_ScriptModule.SetName(pszName);
	g_ScriptModule.QueryInterface(IID_IScriptModule, reinterpret_cast<void**>(pOutInterface));

	return 1;
}

extern "C" APIENTRY
BOOL DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID)
{
	if (dwReason == DLL_PROCESS_ATTACH) {
		// MS says it's not "safe" to call LoadLibrary in DllMain
		// ... eh, it seems to works. So screw 'em.
		HMODULE hHackModule = LoadLibraryA("dromedhack.dll");
		if (hHackModule)
		{
			GetDromedProcAddress pfnGetDromedProcAddress = reinterpret_cast<GetDromedProcAddress>(GetProcAddress(hHackModule, "_GetDromedProcAddress@4"));
			if (pfnGetDromedProcAddress)
				pfnConfigGet = reinterpret_cast<config_get_raw>(pfnGetDromedProcAddress(__CONFIG_GET_RAW));
			FreeLibrary(hHackModule);
		}
	}
	else if (dwReason == DLL_PROCESS_DETACH) {
		if (g_hPyNumbraModule)
			FreeLibrary(g_hPyNumbraModule);
	}
	return TRUE;
}

