/******************************************************************************
 *    ShockGameService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#if (_DARKGAME == 3)
#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<IShockGameSrv>::Name = "ShockGameService";

	const char * 
	ScriptService<IShockGameSrv>::DocStrings[] = {
		"ShockGameService()\n\t"
			"Only available in System Shock 2",
		"DestroyCursorObj()\n\t"
			"Destroys the object currently being used as the cursor.",
		"DestroyInvObj(obj)\n\t"
			"Destroys an object which may be in the host's inventory.",
		"HideInvObj(obj)",
		"SetPlayerPsiPoints(int)\n\t"
			"Modifies the host player's psi-points.",
		"GetPlayerPsiPoints() -> Integer\n\t"
			"Returns the host player's psi-points.",
		"AttachCamera(str)\n\t",
		"CutSceneModeOn(str)\n\t",
		"CutSceneModeOff()\n\t",
		"CreatePlayerPuppet(str=None) -> Integer\n\t",
		"DestroyPlayerPuppet()\n\t",
		"Replicator(obj)\n\t"
			"Pops up the replicator interface for the object.",
		"Container(obj)\n\t"
			"Pops up the container interface for the object.",
		"YorN(obj,prompt)\n\t"
			"Prompts for a \"Yes\" or \"No\" response to a prompt.",
		"Keypad(obj)\n\t"
			"Pops up the keypad interface for the object.",
		"HRM(int,obj,int)\n\t",
		"TechTool(obj)\n\t"
			"Pops up the research status display.",
		"UseLog(obj,int)\n\t",
		"TriggerLog(type,int,int,force) -> Integer\n\t"
			"Emails are 0, logs are 1. force is non-zero to always show.\n\t"
			"The other arguments correspond to the values returned by\n\t"
			"FindLogData.",
		"FindLogData(obj,type) -> Tuple\n\t"
			"Retrieves the log data from the object's properties.",
		"PayNanites(amount) -> Integer\n\t"
			"Attempt to subtract the amount of nanites from the player host.\n\t"
			"Returns non-zero if there aren't enough nanites.\n\t"
			"amount may be negative.",
		"OverlayChange(type,state)\n\t"
			"Sets the state of one of the HUD screens.",
		"Equipped(type) -> PyNumbra.Object\n\t"
			"Type is the Contains type minus 1000.",
		"LevelTransport(name,location,flags)\n\t"
			"Send the player to another level.",
		"CheckLocked(obj,int,frobber) -> Boolean\n\t"
			"Return whether the object is locked for the frobber.",
		"AddText(str,host,time)\n\t"
			"Display a string on the player's screen. host is the object id\n\t"
			"of the network player to send the message to, or 0 to display\n\t"
			"locally. time is milliseconds.",
		"AddTranslatableText(str,res,host,time)\n\t"
			"Display a string from a res file.",
		"AmmoLoad(gun,ammo)\n\t",
		"GetClip(obj) -> Integer\n\t"
			"Returns the size of the object's clip.",
		"AddExp(obj,amount,type)\n\t"
			"Adds cyber modules if type is non-zero.",
		"HasTrait(obj,type) -> Boolean\n\t",
		"HasImplant(obj,type) -> Boolean\n\t",
		"HealObj(obj,amount)\n\t"
			"Adjust the hit-points of the object. amount may be negative.",
		"OverlaySetObj(type,obj)\n\t"
			"Sets the object that will receive messages from one of the HUD\n\t"
			"screens.",
		"Research()\n\t",
		"GetArchetypeName(obj) -> String\n\t"
			"Retrieves the name of the object's archetype.",
		"OverlayOn(int) -> Integer\n\t",
		"FindSpawnPoint(obj,flags) -> PyNumbra.Object\n\t"
			"Locate a spawn point near obj.",
		"CountEcoMatching(type) -> Integer\n\t",
		"GetStat(obj,type) -> Integer\n\t",
		"GetSelectedObj() -> PyNumbra.Object\n\t",
		"AddInvObj(obj) -> Integer\n\t"
			"Puts the object in this host's inventory.",
		"RecalcStats(obj)\n\t"
			"Adjust the state of an object after having modified its properties.",
		"PlayVideo(str)\n\t"
			"Display a movie. Argument is the filename.",
		"ClearRadiation()\n\t",
		"SetPlayerVolume(float)\n\t",
		"RandRange(low,high) -> Integer\n\t"
			"[low,high]",
		"LoadCursor(obj)\n\t"
			"Make an object be the cursor.",
		"AddSpeedControl(name,scale,rot)\n\t",
		"RemoveSpeedControl(name)\n\t",
		"PreventSwap()\n\t"
			"Tells the UI not to change the current selection when an object\n\t"
			"is clicked on. (I think.)",
		"GetDistantSelectedObj() -> PyNumbra.Object\n\t",
		"Equip(type,obj)\n\t",
		"OverlayChangeObj(type,state,obj)\n\t"
			"Changes the state of a HUD screen and sets the object that will\n\t"
			"receive messages from it.",
		"SetObjState(obj,int)\n\t"
			"State 5 is hacked.",
		"RadiationHack()\n\t"
			"Used by the EraseRadiation script from allobjs.",
		"DestroyAllByName(name)\n\t",
		"AddTextObjProp(obj,prop,host,time)\n\t"
			"Display text from an object's property. The property should\n\t"
			"be a label.",
		"DisableAlarmGlobal()\n\t"
			"Turns off all alarms on this level.",
		"Frob(int)\n\t"
			"Make the player frob the currently selected item.",
		"TweqAllByName(name,flags)\n\t",
		"SetExplored(location,explored)\n\t"
			"Change whether a location of the map has been explored.",
		"RemoveFromContainer(obj,container)\n\t",
		"ActivateMap()\n\t"
			"Displays the auto-map.",
		"SimTime() -> Integer\n\t"
			"Returns the amount of simulation time that has elapsed.\n\t"
			"Sim time is roughly equivalent to milliseconds, but not exactly.",
		"StartFadeIn(time,(RGB))\n\t"
			"Make the screen a solid color, then gradually fade to full\n\t"
			"visibility.",
		"StartFadeOut(time,(RGB))\n\t"
			"Gradually transition to a solid color over a given number\n\t"
			"of milliseconds.",
		"GrantPsiPower(obj,type)\n\t"
			"Give a psi power to the object.",
		"ResearchConsume(obj) -> Integer\n\t"
			"Attempt to use the object in the current research process.\n\t"
			"Returns non-zero if the object was used.",
		"PlayerMode(mode)\n\t"
			"Set the player mode.",
		"EndGame()\n\t"
			"Nothing more to do.",
		"AllowDeath() -> Integer\n\t"
			"Returns non-zero if the player dying is not cause to end the game.",
		"AddAlarm(time)\n\t"
			"time is milliseconds.",
		"RemoveAlarm()\n\t"
			"Immediately cancels the alarm.",
		"GetHazardResistance(type) -> Float\n\t"
			"Retrieve the player's resistance to a type of damage.",
		"GetBurnDmg() -> Integer\n\t"
			"Returns the amount of damage done by a psi overload.",
		"PlayerGun() -> PyNumbra.Object\n\t"
			"Get the gun being wielded by the player.",
		"IsPsiActive(type) -> Boolean\n\t"
			"Check whether a psi power is being used.",
		"PsiRadarScan()\n\t",
		"PseudoProjectile(firer,archetype) -> PyNumbra.Object\n\t",
		"WearArmor(obj)\n\t"
			"An object id of 0 will remove the armor.",
		"SetModify(obj,state)\n\t"
			"Change the gun modification state.",
		"Censored() -> Integer\n\t"
			"Returns non-zero if the gore-factor needs to be reduced.",
		"DebriefMode(mode)\n\t",
		"TlucTextAdd(str,res,num)\n\t"
			"Display a res string in a translucent box. The num is appended\n\t"
			"to the name of the string. The translucent box is HUD screen 41.\n\t"
			"Use OverlayChange to hide it.",
		"Mouse(show,int)\n\t"
			"Show or hide the cursor.",
		"RefreshInv()\n\t"
			"Update the state of the host's inventory.",
		"TreasureTable(obj)\n\t"
			"Add the object to the loot list.",
		"OverlayGetObj() -> PyNumbra.Object\n\t",
		"VaporizeInv()\n\t"
			"Clear the contents of the inventory.",
		"ShutoffPsi()\n\t"
			"Deactivates all psi powers.",
		"SetQBHacked(name,val)\n\t"
			"Sets the value of a quest variable.\n\t"
			"This seems to be preferred to QuestService().Set.",
		"GetPlayerMaxPsiPoints() -> Integer\n\t"
			"Retrieve the current maximum value for the player's psi.",
		"SetLogTime(int,type,int)\n\t",
		"AddTranslatableTextInt(str,res,host,param,time)\n\t"
			"Display a res string with an integer parameter.\n\t"
			"A single '%d' in the string will be replaced with the param.",
		"ZeroControls(obj,int)\n\t",
		"SetSelectedPsiPower(type)\n\t",
		"ValidGun(obj) -> Integer\n\t"
			"Returns non-zero if the object is a type of gun.",
		"AddTranslatableTextIndexInt(str,res,host,num,param,time)\n\t"
			"Display an indexed res string with an integer parameter.\n\t"
			"num is appended to the string name.",
		"IsAlarmActive() -> Boolean\n\t",
		"SlayAllByName(name)\n\t",
		"NoMove(freeze)\n\t"
			"Prevents the player from moving if freeze is non-zero. Zero will\n\t"
			"allow movement again. Turning and free-look are still allowed,\n\t"
			"as is the user-interface.",
		"PlayerModeSimple(int)\n\t",
		"UpdateMovingTerrainVelocity(object,object,float)\n\t",
		"MouseCursor() -> Integer\n\t"
			"Returns non-zero if the mouse cursor is visible.",
	};

	const PyMethodDef 
	ScriptService<IShockGameSrv>::Methods[] = {
		{	"DestroyCursorObj",
			reinterpret_cast<PyCFunction>(DestroyCursorObj),
			METH_NOARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"DestroyInvObj",
			reinterpret_cast<PyCFunction>(DestroyInvObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"HideInvObj",
			reinterpret_cast<PyCFunction>(HideInvObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"SetPlayerPsiPoints",
			reinterpret_cast<PyCFunction>(SetPlayerPsiPoints),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"GetPlayerPsiPoints",
			reinterpret_cast<PyCFunction>(GetPlayerPsiPoints),
			METH_NOARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"AttachCamera",
			reinterpret_cast<PyCFunction>(AttachCamera),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"CutSceneModeOn",
			reinterpret_cast<PyCFunction>(CutSceneModeOn),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"CutSceneModeOff",
			reinterpret_cast<PyCFunction>(CutSceneModeOff),
			METH_NOARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"CreatePlayerPuppet",
			reinterpret_cast<PyCFunction>(CreatePlayerPuppet),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"DestroyPlayerPuppet",
			reinterpret_cast<PyCFunction>(DestroyPlayerPuppet),
			METH_NOARGS,
			const_cast<char*>(DocStrings[10])
		},
		{	"Replicator",
			reinterpret_cast<PyCFunction>(Replicator),
			METH_VARARGS,
			const_cast<char*>(DocStrings[11])
		},
		{	"Container",
			reinterpret_cast<PyCFunction>(Container),
			METH_VARARGS,
			const_cast<char*>(DocStrings[12])
		},
		{	"YorN",
			reinterpret_cast<PyCFunction>(YorN),
			METH_VARARGS,
			const_cast<char*>(DocStrings[13])
		},
		{	"Keypad",
			reinterpret_cast<PyCFunction>(Keypad),
			METH_VARARGS,
			const_cast<char*>(DocStrings[14])
		},
		{	"HRM",
			reinterpret_cast<PyCFunction>(HRM),
			METH_VARARGS,
			const_cast<char*>(DocStrings[15])
		},
		{	"TechTool",
			reinterpret_cast<PyCFunction>(TechTool),
			METH_VARARGS,
			const_cast<char*>(DocStrings[16])
		},
		{	"UseLog",
			reinterpret_cast<PyCFunction>(UseLog),
			METH_VARARGS,
			const_cast<char*>(DocStrings[17])
		},
		{	"TriggerLog",
			reinterpret_cast<PyCFunction>(TriggerLog),
			METH_VARARGS,
			const_cast<char*>(DocStrings[18])
		},
		{	"FindLogData",
			reinterpret_cast<PyCFunction>(FindLogData),
			METH_VARARGS,
			const_cast<char*>(DocStrings[19])
		},
		{	"PayNanites",
			reinterpret_cast<PyCFunction>(PayNanites),
			METH_VARARGS,
			const_cast<char*>(DocStrings[20])
		},
		{	"OverlayChange",
			reinterpret_cast<PyCFunction>(OverlayChange),
			METH_VARARGS,
			const_cast<char*>(DocStrings[21])
		},
		{	"Equipped",
			reinterpret_cast<PyCFunction>(Equipped),
			METH_VARARGS,
			const_cast<char*>(DocStrings[22])
		},
		{	"LevelTransport",
			reinterpret_cast<PyCFunction>(LevelTransport),
			METH_VARARGS,
			const_cast<char*>(DocStrings[23])
		},
		{	"CheckLocked",
			reinterpret_cast<PyCFunction>(CheckLocked),
			METH_VARARGS,
			const_cast<char*>(DocStrings[24])
		},
		{	"AddText",
			reinterpret_cast<PyCFunction>(AddText),
			METH_VARARGS,
			const_cast<char*>(DocStrings[25])
		},
		{	"AddTranslatableText",
			reinterpret_cast<PyCFunction>(AddTranslatableText),
			METH_VARARGS,
			const_cast<char*>(DocStrings[26])
		},
		{	"AmmoLoad",
			reinterpret_cast<PyCFunction>(AmmoLoad),
			METH_VARARGS,
			const_cast<char*>(DocStrings[27])
		},
		{	"GetClip",
			reinterpret_cast<PyCFunction>(GetClip),
			METH_VARARGS,
			const_cast<char*>(DocStrings[28])
		},
		{	"AddExp",
			reinterpret_cast<PyCFunction>(AddExp),
			METH_VARARGS,
			const_cast<char*>(DocStrings[29])
		},
		{	"HasTrait",
			reinterpret_cast<PyCFunction>(HasTrait),
			METH_VARARGS,
			const_cast<char*>(DocStrings[30])
		},
		{	"HasImplant",
			reinterpret_cast<PyCFunction>(HasImplant),
			METH_VARARGS,
			const_cast<char*>(DocStrings[31])
		},
		{	"HealObj",
			reinterpret_cast<PyCFunction>(HealObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[32])
		},
		{	"OverlaySetObj",
			reinterpret_cast<PyCFunction>(OverlaySetObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[33])
		},
		{	"Research",
			reinterpret_cast<PyCFunction>(Research),
			METH_NOARGS,
			const_cast<char*>(DocStrings[34])
		},
		{	"GetArchetypeName",
			reinterpret_cast<PyCFunction>(GetArchetypeName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[35])
		},
		{	"OverlayOn",
			reinterpret_cast<PyCFunction>(OverlayOn),
			METH_VARARGS,
			const_cast<char*>(DocStrings[36])
		},
		{	"FindSpawnPoint",
			reinterpret_cast<PyCFunction>(FindSpawnPoint),
			METH_VARARGS,
			const_cast<char*>(DocStrings[37])
		},
		{	"CountEcoMatching",
			reinterpret_cast<PyCFunction>(CountEcoMatching),
			METH_VARARGS,
			const_cast<char*>(DocStrings[38])
		},
		{	"GetStat",
			reinterpret_cast<PyCFunction>(GetStat),
			METH_VARARGS,
			const_cast<char*>(DocStrings[39])
		},
		{	"GetSelectedObj",
			reinterpret_cast<PyCFunction>(GetSelectedObj),
			METH_NOARGS,
			const_cast<char*>(DocStrings[40])
		},
		{	"AddInvObj",
			reinterpret_cast<PyCFunction>(AddInvObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[41])
		},
		{	"RecalcStats",
			reinterpret_cast<PyCFunction>(RecalcStats),
			METH_VARARGS,
			const_cast<char*>(DocStrings[42])
		},
		{	"PlayVideo",
			reinterpret_cast<PyCFunction>(PlayVideo),
			METH_VARARGS,
			const_cast<char*>(DocStrings[43])
		},
		{	"ClearRadiation",
			reinterpret_cast<PyCFunction>(ClearRadiation),
			METH_NOARGS,
			const_cast<char*>(DocStrings[44])
		},
		{	"SetPlayerVolume",
			reinterpret_cast<PyCFunction>(SetPlayerVolume),
			METH_VARARGS,
			const_cast<char*>(DocStrings[45])
		},
		{	"RandRange",
			reinterpret_cast<PyCFunction>(RandRange),
			METH_VARARGS,
			const_cast<char*>(DocStrings[46])
		},
		{	"LoadCursor",
			reinterpret_cast<PyCFunction>(LoadCursor),
			METH_VARARGS,
			const_cast<char*>(DocStrings[47])
		},
		{	"AddSpeedControl",
			reinterpret_cast<PyCFunction>(AddSpeedControl),
			METH_VARARGS,
			const_cast<char*>(DocStrings[48])
		},
		{	"RemoveSpeedControl",
			reinterpret_cast<PyCFunction>(RemoveSpeedControl),
			METH_VARARGS,
			const_cast<char*>(DocStrings[49])
		},
		{	"PreventSwap",
			reinterpret_cast<PyCFunction>(PreventSwap),
			METH_NOARGS,
			const_cast<char*>(DocStrings[50])
		},
		{	"GetDistantSelectedObj",
			reinterpret_cast<PyCFunction>(GetDistantSelectedObj),
			METH_NOARGS,
			const_cast<char*>(DocStrings[51])
		},
		{	"Equip",
			reinterpret_cast<PyCFunction>(Equip),
			METH_VARARGS,
			const_cast<char*>(DocStrings[52])
		},
		{	"OverlayChangeObj",
			reinterpret_cast<PyCFunction>(OverlayChangeObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[53])
		},
		{	"SetObjState",
			reinterpret_cast<PyCFunction>(SetObjState),
			METH_VARARGS,
			const_cast<char*>(DocStrings[54])
		},
		{	"RadiationHack",
			reinterpret_cast<PyCFunction>(RadiationHack),
			METH_NOARGS,
			const_cast<char*>(DocStrings[55])
		},
		{	"DestroyAllByName",
			reinterpret_cast<PyCFunction>(DestroyAllByName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[56])
		},
		{	"AddTextObjProp",
			reinterpret_cast<PyCFunction>(AddTextObjProp),
			METH_VARARGS,
			const_cast<char*>(DocStrings[57])
		},
		{	"DisableAlarmGlobal",
			reinterpret_cast<PyCFunction>(DisableAlarmGlobal),
			METH_NOARGS,
			const_cast<char*>(DocStrings[58])
		},
		{	"Frob",
			reinterpret_cast<PyCFunction>(Frob),
			METH_VARARGS,
			const_cast<char*>(DocStrings[59])
		},
		{	"TweqAllByName",
			reinterpret_cast<PyCFunction>(TweqAllByName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[60])
		},
		{	"SetExplored",
			reinterpret_cast<PyCFunction>(SetExplored),
			METH_VARARGS,
			const_cast<char*>(DocStrings[61])
		},
		{	"RemoveFromContainer",
			reinterpret_cast<PyCFunction>(RemoveFromContainer),
			METH_VARARGS,
			const_cast<char*>(DocStrings[62])
		},
		{	"ActivateMap",
			reinterpret_cast<PyCFunction>(ActivateMap),
			METH_NOARGS,
			const_cast<char*>(DocStrings[63])
		},
		{	"SimTime",
			reinterpret_cast<PyCFunction>(SimTime),
			METH_NOARGS,
			const_cast<char*>(DocStrings[64])
		},
		{	"StartFadeIn",
			reinterpret_cast<PyCFunction>(StartFadeIn),
			METH_VARARGS,
			const_cast<char*>(DocStrings[65])
		},
		{	"StartFadeOut",
			reinterpret_cast<PyCFunction>(StartFadeOut),
			METH_VARARGS,
			const_cast<char*>(DocStrings[66])
		},
		{	"GrantPsiPower",
			reinterpret_cast<PyCFunction>(GrantPsiPower),
			METH_VARARGS,
			const_cast<char*>(DocStrings[67])
		},
		{	"ResearchConsume",
			reinterpret_cast<PyCFunction>(ResearchConsume),
			METH_VARARGS,
			const_cast<char*>(DocStrings[68])
		},
		{	"PlayerMode",
			reinterpret_cast<PyCFunction>(PlayerMode),
			METH_VARARGS,
			const_cast<char*>(DocStrings[69])
		},
		{	"EndGame",
			reinterpret_cast<PyCFunction>(EndGame),
			METH_NOARGS,
			const_cast<char*>(DocStrings[70])
		},
		{	"AllowDeath",
			reinterpret_cast<PyCFunction>(AllowDeath),
			METH_NOARGS,
			const_cast<char*>(DocStrings[71])
		},
		{	"AddAlarm",
			reinterpret_cast<PyCFunction>(AddAlarm),
			METH_VARARGS,
			const_cast<char*>(DocStrings[72])
		},
		{	"RemoveAlarm",
			reinterpret_cast<PyCFunction>(RemoveAlarm),
			METH_NOARGS,
			const_cast<char*>(DocStrings[73])
		},
		{	"GetHazardResistance",
			reinterpret_cast<PyCFunction>(GetHazardResistance),
			METH_VARARGS,
			const_cast<char*>(DocStrings[74])
		},
		{	"GetBurnDmg",
			reinterpret_cast<PyCFunction>(GetBurnDmg),
			METH_NOARGS,
			const_cast<char*>(DocStrings[75])
		},
		{	"PlayerGun",
			reinterpret_cast<PyCFunction>(PlayerGun),
			METH_NOARGS,
			const_cast<char*>(DocStrings[76])
		},
		{	"IsPsiActive",
			reinterpret_cast<PyCFunction>(IsPsiActive),
			METH_VARARGS,
			const_cast<char*>(DocStrings[77])
		},
		{	"PsiRadarScan",
			reinterpret_cast<PyCFunction>(PsiRadarScan),
			METH_NOARGS,
			const_cast<char*>(DocStrings[78])
		},
		{	"PseudoProjectile",
			reinterpret_cast<PyCFunction>(PseudoProjectile),
			METH_VARARGS,
			const_cast<char*>(DocStrings[79])
		},
		{	"WearArmor",
			reinterpret_cast<PyCFunction>(WearArmor),
			METH_VARARGS,
			const_cast<char*>(DocStrings[80])
		},
		{	"SetModify",
			reinterpret_cast<PyCFunction>(SetModify),
			METH_VARARGS,
			const_cast<char*>(DocStrings[81])
		},
		{	"Censored",
			reinterpret_cast<PyCFunction>(Censored),
			METH_NOARGS,
			const_cast<char*>(DocStrings[82])
		},
		{	"DebriefMode",
			reinterpret_cast<PyCFunction>(DebriefMode),
			METH_VARARGS,
			const_cast<char*>(DocStrings[83])
		},
		{	"TlucTextAdd",
			reinterpret_cast<PyCFunction>(TlucTextAdd),
			METH_VARARGS,
			const_cast<char*>(DocStrings[84])
		},
		{	"Mouse",
			reinterpret_cast<PyCFunction>(Mouse),
			METH_VARARGS,
			const_cast<char*>(DocStrings[85])
		},
		{	"RefreshInv",
			reinterpret_cast<PyCFunction>(RefreshInv),
			METH_NOARGS,
			const_cast<char*>(DocStrings[86])
		},
		{	"TreasureTable",
			reinterpret_cast<PyCFunction>(TreasureTable),
			METH_VARARGS,
			const_cast<char*>(DocStrings[87])
		},
		{	"OverlayGetObj",
			reinterpret_cast<PyCFunction>(OverlayGetObj),
			METH_NOARGS,
			const_cast<char*>(DocStrings[88])
		},
		{	"VaporizeInv",
			reinterpret_cast<PyCFunction>(VaporizeInv),
			METH_NOARGS,
			const_cast<char*>(DocStrings[89])
		},
		{	"ShutoffPsi",
			reinterpret_cast<PyCFunction>(ShutoffPsi),
			METH_NOARGS,
			const_cast<char*>(DocStrings[90])
		},
		{	"SetQBHacked",
			reinterpret_cast<PyCFunction>(SetQBHacked),
			METH_VARARGS,
			const_cast<char*>(DocStrings[91])
		},
		{	"GetPlayerMaxPsiPoints",
			reinterpret_cast<PyCFunction>(GetPlayerMaxPsiPoints),
			METH_NOARGS,
			const_cast<char*>(DocStrings[92])
		},
		{	"SetLogTime",
			reinterpret_cast<PyCFunction>(SetLogTime),
			METH_VARARGS,
			const_cast<char*>(DocStrings[93])
		},
		{	"AddTranslatableTextInt",
			reinterpret_cast<PyCFunction>(AddTranslatableTextInt),
			METH_VARARGS,
			const_cast<char*>(DocStrings[94])
		},
		{	"ZeroControls",
			reinterpret_cast<PyCFunction>(ZeroControls),
			METH_VARARGS,
			const_cast<char*>(DocStrings[95])
		},
		{	"SetSelectedPsiPower",
			reinterpret_cast<PyCFunction>(SetSelectedPsiPower),
			METH_VARARGS,
			const_cast<char*>(DocStrings[96])
		},
		{	"ValidGun",
			reinterpret_cast<PyCFunction>(ValidGun),
			METH_VARARGS,
			const_cast<char*>(DocStrings[97])
		},
		{	"AddTranslatableTextIndexInt",
			reinterpret_cast<PyCFunction>(AddTranslatableTextIndexInt),
			METH_VARARGS,
			const_cast<char*>(DocStrings[98])
		},
		{	"IsAlarmActive",
			reinterpret_cast<PyCFunction>(IsAlarmActive),
			METH_NOARGS,
			const_cast<char*>(DocStrings[99])
		},
		{	"SlayAllByName",
			reinterpret_cast<PyCFunction>(SlayAllByName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[100])
		},
		{	"NoMove",
			reinterpret_cast<PyCFunction>(NoMove),
			METH_VARARGS,
			const_cast<char*>(DocStrings[101])
		},
		{	"PlayerModeSimple",
			reinterpret_cast<PyCFunction>(PlayerModeSimple),
			METH_VARARGS,
			const_cast<char*>(DocStrings[102])
		},
		{	"UpdateMovingTerrainVelocity",
			reinterpret_cast<PyCFunction>(UpdateMovingTerrainVelocity),
			METH_VARARGS,
			const_cast<char*>(DocStrings[103])
		},
		{	"MouseCursor",
			reinterpret_cast<PyCFunction>(MouseCursor),
			METH_NOARGS,
			const_cast<char*>(DocStrings[104])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IShockGameSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IShockGameSrv>::Name),
		sizeof(ScriptServiceObject<IShockGameSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IShockGameSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IShockGameSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IShockGameSrv> * 
	ScriptService<IShockGameSrv>::NewObject(PyObject *)
	{
		IShockGameSrv *iface = static_cast<IShockGameSrv*>(g_pScriptManager->GetService(IID_IShockGameScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IShockGameSrv> *obj = PyObject_New(ScriptServiceObject<IShockGameSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IShockGameSrv>::DestroyCursorObj(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->DestroyCursorObj());
	}

	PyObject * 
	ScriptService<IShockGameSrv>::DestroyInvObj(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:DestroyInvObj", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->DestroyInvObj(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::HideInvObj(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:HideInvObj", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->HideInvObj(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetPlayerPsiPoints(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int points;
		if (!PyArg_ParseTuple(args, "i:SetPlayerPsiPoints", &points))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->HideInvObj(points));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetPlayerPsiPoints(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->GetPlayerPsiPoints());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AttachCamera(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *s;
		if (!PyArg_ParseTuple(args, "s:AttachCamera", &s))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AttachCamera(s));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::CutSceneModeOn(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *s;
		if (!PyArg_ParseTuple(args, "s:CutSceneModeOn", &s))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->CutSceneModeOn(s));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::CutSceneModeOff(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->CutSceneModeOff());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::CreatePlayerPuppet(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name = NULL;
		if (!PyArg_ParseTuple(args, "|z:CreatePlayerPuppet", &name))
		{
			return NULL;
		}
		int ret;
		if (name)
			ret = self->m_pIFace->CreatePlayerPuppet(name);
		else
			ret = self->m_pIFace->CreatePlayerPuppet();
		return PyInt_FromLong(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::DestroyPlayerPuppet(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->DestroyPlayerPuppet());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Replicator(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Replicator", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Replicator(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Container(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Container", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Container(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::YorN(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *msg;
		object obj;
		if (!PyArg_ParseTuple(args, "O&s:YorN", cvtObject, &obj, &msg))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->YorN(obj,msg));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Keypad(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Keypad", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Keypad(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::HRM(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int n1,n2;
		if (!PyArg_ParseTuple(args, "iO&i:HRM", &n1, cvtObject, &obj, &n2))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->HRM(n1,obj,n2));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::TechTool(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:TechTool", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->TechTool(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::UseLog(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int n;
		if (!PyArg_ParseTuple(args, "O&i:UseLog", cvtObject, &obj, &n))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->UseLog(obj,n));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::TriggerLog(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int log1, log2;
		int data1, data2;
		if (!PyArg_ParseTuple(args, "iiii:TriggerLog", &log1, &data1, &data2, &log2))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->TriggerLog(log1,data1,data2,log2));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::FindLogData(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int log;
		int data1, data2;
		if (!PyArg_ParseTuple(args, "O&i:FindLogData", cvtObject, &obj, &log))
		{
			return NULL;
		}
		if (self->m_pIFace->FindLogData(obj,log,&data1,&data2))
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return Py_BuildValue("(ii)", data1, data2);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PayNanites(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int val;
		if (!PyArg_ParseTuple(args, "i:PayNanites", &val))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->PayNanites(val));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::OverlayChange(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int type, mode;
		if (!PyArg_ParseTuple(args, "ii:OverlayChange", &type, &mode))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->OverlayChange(type,mode));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Equipped(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int type;
		if (!PyArg_ParseTuple(args, "i:Equipped", &type))
		{
			return NULL;
		}
		object ret;
		self->m_pIFace->Equipped(ret,type);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::LevelTransport(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *level;
		int loc;
		unsigned int flags;
		if (!PyArg_ParseTuple(args, "sii:LevelTransport", &level, &loc, &flags))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->LevelTransport(level,loc,flags));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::CheckLocked(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj, frobber;
		int n;
		if (!PyArg_ParseTuple(args, "O&iO&:CheckLocked", cvtObject, &obj, &n, cvtObject, &frobber))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->CheckLocked(obj,n,frobber)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddText(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *text;
		object host;
		int t;
		if (!PyArg_ParseTuple(args, "sO&i:AddText", &text, cvtObject, &host, &t))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddText(text,host,t));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddTranslatableText(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object host;
		const char *text, *s;
		int t;
		if (!PyArg_ParseTuple(args, "ssO&i:AddTranslatableText", &text, &s, cvtObject, &host, &t))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddTranslatableText(text,s,host,t));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AmmoLoad(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object gun,clip;
		if (!PyArg_ParseTuple(args, "O&O&:AmmoLoad", cvtObject, &gun, cvtObject, &clip))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AmmoLoad(gun,clip));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetClip(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object gun;
		if (!PyArg_ParseTuple(args, "O&:GetClip", cvtObject, &gun))
		{
			return NULL;
		}
		// Is this supposed to be an object id?
		return PyInt_FromLong(self->m_pIFace->GetClip(gun));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddExp(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int val, type;
		if (!PyArg_ParseTuple(args, "O&ii:AddExp", cvtObject, &obj, &val, &type))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddExp(obj,val,type));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::HasTrait(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int type;
		if (!PyArg_ParseTuple(args, "O&i:HasTrait", cvtObject, &obj, &type))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->HasTrait(obj,type)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::HasImplant(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int type;
		if (!PyArg_ParseTuple(args, "O&i:HasImplant", cvtObject, &obj, &type))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->HasImplant(obj,type)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::HealObj(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int val;
		if (!PyArg_ParseTuple(args, "O&i:HealObj", cvtObject, &obj, &val))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->HealObj(obj,val));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::OverlaySetObj(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int type;
		if (!PyArg_ParseTuple(args, "iO&:OverlaySetObj", &type, cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->OverlaySetObj(type,obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Research(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->Research());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetArchetypeName(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:GetArchetypeName", cvtObject, &obj))
		{
			return NULL;
		}
		cScrStr ret;
		self->m_pIFace->GetArchetypeName(ret,obj);
		return PyString_FromString(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::OverlayOn(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int type;
		if (!PyArg_ParseTuple(args, "i:OverlayOn", &type))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->OverlayOn(type));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::FindSpawnPoint(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		unsigned int flags;
		if (!PyArg_ParseTuple(args, "O&i:FindSpawnPoint", cvtObject, &obj, &flags))
		{
			return NULL;
		}
		object ret;
		self->m_pIFace->FindSpawnPoint(ret,obj,flags);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::CountEcoMatching(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int type;
		if (!PyArg_ParseTuple(args, "i:CountEcoMatching", &type))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->CountEcoMatching(type));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetStat(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int type;
		if (!PyArg_ParseTuple(args, "O&i:GetStat", cvtObject, &obj, &type))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->GetStat(obj,type));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetSelectedObj(ScriptServiceObject<IShockGameSrv> *self)
	{
		object ret;
		self->m_pIFace->GetSelectedObj(ret);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddInvObj(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:AddInvObj", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddInvObj(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RecalcStats(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:RecalcStats", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->RecalcStats(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PlayVideo(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:PlayVideo", &name))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->PlayVideo(name));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::ClearRadiation(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->ClearRadiation());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetPlayerVolume(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		float volume;
		if (!PyArg_ParseTuple(args, "f:SetPlayerVolume", &volume))
		{
			return NULL;
		}
		self->m_pIFace->SetPlayerVolume(volume);
		Py_INCREF(Py_None);
		return Py_None;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RandRange(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int lo,hi;
		if (!PyArg_ParseTuple(args, "ii:RandRange", &lo, &hi))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->RandRange(lo,hi));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::LoadCursor(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:LoadCursor", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->LoadCursor(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddSpeedControl(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		float speed_scale, turn_scale;
		if (!PyArg_ParseTuple(args, "sff:AddSpeedControl", &name, &speed_scale, &turn_scale))
		{
			return NULL;
		}
		self->m_pIFace->AddSpeedControl(name,speed_scale,turn_scale);
		Py_INCREF(Py_None);
		return Py_None;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RemoveSpeedControl(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:RemoveSpeedControl", &name))
		{
			return NULL;
		}
		self->m_pIFace->RemoveSpeedControl(name);
		Py_INCREF(Py_None);
		return Py_None;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PreventSwap(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->PreventSwap());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetDistantSelectedObj(ScriptServiceObject<IShockGameSrv> *self)
	{
		object ret;
		self->m_pIFace->GetDistantSelectedObj(ret);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Equip(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int type;
		if (!PyArg_ParseTuple(args, "iO&:Equip", &type, cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Equip(type,obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::OverlayChangeObj(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int type, state;
		if (!PyArg_ParseTuple(args, "iiO&:OverlayChangeObj", &type, &state, cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->OverlayChangeObj(type,state,obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetObjState(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int state;
		if (!PyArg_ParseTuple(args, "O&i:SetObjState", cvtObject, &obj, &state))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetObjState(obj,state));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RadiationHack(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->RadiationHack());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::DestroyAllByName(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:DestroyAllByName", &name))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->DestroyAllByName(name));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddTextObjProp(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *prop;
		object obj, host;
		int t;
		if (!PyArg_ParseTuple(args, "O&sO&i:AddTextObjProp", cvtObject, &obj, &prop, cvtObject, &host, &t))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddTextObjProp(obj,prop,host,t));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::DisableAlarmGlobal(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->DisableAlarmGlobal());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Frob(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int n;
		if (!PyArg_ParseTuple(args, "i:Frob", &n))
		{
			return NULL;
		}
		self->m_pIFace->Frob(n);
		Py_INCREF(Py_None);
		return Py_None;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::TweqAllByName(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		int flags;
		if (!PyArg_ParseTuple(args, "si:TweqAllByName", &name, &flags))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->TweqAllByName(name,flags));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetExplored(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int loc;
		char b;
		if (!PyArg_ParseTuple(args, "ib:SetExplored", &loc, &b))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetExplored(loc,b));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RemoveFromContainer(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj, cont;
		if (!PyArg_ParseTuple(args, "O&O&:RemoveFromContainer", cvtObject, &obj, cvtObject, &cont))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->RemoveFromContainer(obj,cont));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::ActivateMap(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->ActivateMap());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SimTime(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->SimTime());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::StartFadeIn(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		unsigned char r,g,b;
		int ms;
		if (!PyArg_ParseTuple(args, "i(bbb):StartFadeIn", &ms, &r, &g, &b))
		{
			return NULL;
		}
		self->m_pIFace->StartFadeIn(ms,r,g,b);
		Py_INCREF(Py_None);
		return Py_None;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::StartFadeOut(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		unsigned char r,g,b;
		int ms;
		if (!PyArg_ParseTuple(args, "i(bbb):StartFadeOut", &ms, &r, &g, &b))
		{
			return NULL;
		}
		self->m_pIFace->StartFadeOut(ms,r,g,b);
		Py_INCREF(Py_None);
		return Py_None;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GrantPsiPower(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int psi;
		if (!PyArg_ParseTuple(args, "O&i:GrantPsiPower", cvtObject, &obj, &psi))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->GrantPsiPower(obj,psi));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::ResearchConsume(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:ResearchConsume", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->ResearchConsume(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PlayerMode(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int mode;
		if (!PyArg_ParseTuple(args, "i:PlayerMode", &mode))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->PlayerMode(mode));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::EndGame(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->EndGame());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AllowDeath(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->AllowDeath());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddAlarm(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int n;
		if (!PyArg_ParseTuple(args, "i:AddAlarm", &n))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->PlayerMode(n));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RemoveAlarm(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->RemoveAlarm());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetHazardResistance(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int type;
		if (!PyArg_ParseTuple(args, "i:GetHazardResistance", &type))
		{
			return NULL;
		}
		return PyFloat_FromDouble(self->m_pIFace->GetHazardResistance(type));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetBurnDmg(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->GetBurnDmg());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PlayerGun(ScriptServiceObject<IShockGameSrv> *self)
	{
		object ret;
		self->m_pIFace->PlayerGun(ret);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::IsPsiActive(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int psi;
		if (!PyArg_ParseTuple(args, "i:IsPsiActive", &psi))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsPsiActive(psi)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PsiRadarScan(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->PsiRadarScan());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PseudoProjectile(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj1, obj2;
		if (!PyArg_ParseTuple(args, "O&O&:PseudoProjectile", cvtObject, &obj1, cvtObject, &obj2))
		{
			return NULL;
		}
		object ret;
		self->m_pIFace->PseudoProjectile(ret,obj1,obj2);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::WearArmor(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:WearArmor", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->WearArmor(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetModify(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int val;
		if (!PyArg_ParseTuple(args, "O&i:SetModify", cvtObject, &obj, &val))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetModify(obj,val));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Censored(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->Censored());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::DebriefMode(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int n;
		if (!PyArg_ParseTuple(args, "i:DebriefMode", &n))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->DebriefMode(n));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::TlucTextAdd(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		char *s, *r;
		int n;
		if (!PyArg_ParseTuple(args, "ssi:TlucTextAdd", &s, &r, &n))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->TlucTextAdd(s,r,n));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::Mouse(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int x,y;
		if (!PyArg_ParseTuple(args, "ii:Mouse", &x, &y))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Mouse(x,y));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::RefreshInv(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->RefreshInv());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::TreasureTable(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:TreasureTable", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->TreasureTable(obj));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::OverlayGetObj(ScriptServiceObject<IShockGameSrv> *self)
	{
		object ret;
		self->m_pIFace->OverlayGetObj(ret);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::VaporizeInv(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->VaporizeInv());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::ShutoffPsi(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->ShutoffPsi());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetQBHacked(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		int val;
		if (!PyArg_ParseTuple(args, "si:SetQBHacked", &name, &val))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetQBHacked(name,val));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::GetPlayerMaxPsiPoints(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->GetPlayerMaxPsiPoints());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetLogTime(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int log1, log2, time;
		if (!PyArg_ParseTuple(args, "iii:SetLogTime", &log1, &log2, &time))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetLogTime(log1,log2,time));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddTranslatableTextInt(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *s, *r;
		object host;
		int n, t;
		if (!PyArg_ParseTuple(args, "ssO&ii:AddTranslatableTextInt", &s, &r, cvtObject, &host, &n, &t))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddTranslatableTextInt(s,r,host,n,t));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::ZeroControls(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		int n;
		if (!PyArg_ParseTuple(args, "O&i:ZeroControls", cvtObject, &obj, &n))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->ZeroControls(obj,n));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SetSelectedPsiPower(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int psi;
		if (!PyArg_ParseTuple(args, "i:SetSelectedPsiPower", &psi))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetSelectedPsiPower(psi));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::ValidGun(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:ValidGun", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->ValidGun(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::AddTranslatableTextIndexInt(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *s, *r;
		object host;
		int i, n, t;
		if (!PyArg_ParseTuple(args, "ssO&iii:AddTranslatableTextIndexInt", &s, &r, cvtObject, &host, &i, &n, &t))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->AddTranslatableTextIndexInt(s,r,host,i,n,t));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::IsAlarmActive(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->IsAlarmActive());
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::SlayAllByName(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:SlayAllByName", &name))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SlayAllByName(name));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::NoMove(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int n;
		if (!PyArg_ParseTuple(args, "i:NoMove", &n))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->NoMove(n));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::PlayerModeSimple(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		int mode;
		if (!PyArg_ParseTuple(args, "i:PlayerModeSimple", &mode))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->PlayerModeSimple(mode));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::UpdateMovingTerrainVelocity(ScriptServiceObject<IShockGameSrv> *self, PyObject *args)
	{
		object obj, terr;
		float speed;
		if (!PyArg_ParseTuple(args, "O&O&f:UpdateMovingTerrainVelocity", cvtObject, &obj, cvtObject, &terr, &speed))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->UpdateMovingTerrainVelocity(obj,terr,speed));
	}
	
	PyObject * 
	ScriptService<IShockGameSrv>::MouseCursor(ScriptServiceObject<IShockGameSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->MouseCursor());
	}


} // namespace PyNumbra

#endif
