/******************************************************************************
 *    ShockObjService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#if (_DARKGAME == 3)
#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<IShockObjSrv>::Name = "ShockObjService";

	const char * 
	ScriptService<IShockObjSrv>::DocStrings[] = {
		"ShockObjService()\n\t"
			"Only available in System Shock 2",
		"FindScriptDonor(obj,name) -> Object\n\t"
			"Returns the archetype from which the script name is inherited from.",
	};

	const PyMethodDef 
	ScriptService<IShockObjSrv>::Methods[] = {
		{	"FindScriptDonor",
			reinterpret_cast<PyCFunction>(FindScriptDonor),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IShockObjSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IShockObjSrv>::Name),
		sizeof(ScriptServiceObject<IShockObjSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IShockObjSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IShockObjSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IShockObjSrv> * 
	ScriptService<IShockObjSrv>::NewObject(PyObject *)
	{
		IShockObjSrv *iface = static_cast<IShockObjSrv*>(g_pScriptManager->GetService(IID_IShockObjScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IShockObjSrv> *obj = PyObject_New(ScriptServiceObject<IShockObjSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IShockObjSrv>::FindScriptDonor(ScriptServiceObject<IShockObjSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		if (!PyArg_ParseTuple(args, "O&s:FindScriptDonor", cvtObject, &obj, &name))
		{
			return NULL;
		}
		int ret;
		ret = self->m_pIFace->FindScriptDonor(obj,name);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(object(ret));
	}


} // namespace PyNumbra

#endif
