/******************************************************************************
 *    ShockWeaponService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#if (_DARKGAME == 3)
#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<IShockWeaponSrv>::Name = "ShockWeaponService";

	const char * 
	ScriptService<IShockWeaponSrv>::DocStrings[] = {
		"ShockWeaponService()\n\t"
			"Only available in System Shock 2",
		"SetWeaponModel(obj)",
		"GetWeaponModel() -> PyNumbra.Object",
		"TargetScan(obj) -> PyNumbra.Object",
		"Home(obj,obj)",
		"DestroyMelee(obj)",
	};

	const PyMethodDef 
	ScriptService<IShockWeaponSrv>::Methods[] = {
		{	"SetWeaponModel",
			reinterpret_cast<PyCFunction>(SetWeaponModel),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"GetWeaponModel",
			reinterpret_cast<PyCFunction>(GetWeaponModel),
			METH_NOARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"TargetScan",
			reinterpret_cast<PyCFunction>(TargetScan),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"Home",
			reinterpret_cast<PyCFunction>(Home),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"DestroyMelee",
			reinterpret_cast<PyCFunction>(DestroyMelee),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IShockWeaponSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IShockWeaponSrv>::Name),
		sizeof(ScriptServiceObject<IShockWeaponSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IShockWeaponSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IShockWeaponSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IShockWeaponSrv> * 
	ScriptService<IShockWeaponSrv>::NewObject(PyObject *)
	{
		IShockWeaponSrv *iface = static_cast<IShockWeaponSrv*>(g_pScriptManager->GetService(IID_IShockWeaponScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IShockWeaponSrv> *obj = PyObject_New(ScriptServiceObject<IShockWeaponSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IShockWeaponSrv>::SetWeaponModel(ScriptServiceObject<IShockWeaponSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:SetWeaponModel", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->SetWeaponModel(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IShockWeaponSrv>::GetWeaponModel(ScriptServiceObject<IShockWeaponSrv> *self)
	{
		object ret;
		self->m_pIFace->GetWeaponModel(ret);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}

	PyObject * 
	ScriptService<IShockWeaponSrv>::TargetScan(ScriptServiceObject<IShockWeaponSrv> *self, PyObject *args)
	{
		object obj;
		object ret;
		if (!PyArg_ParseTuple(args, "O&:TargetScan", cvtObject, &obj))
		{
			return NULL;
		}
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		self->m_pIFace->TargetScan(ret,obj);
		return ObjectObject::New(ret);
	}

	PyObject * 
	ScriptService<IShockWeaponSrv>::Home(ScriptServiceObject<IShockWeaponSrv> *self, PyObject *args)
	{
		object obj1, obj2;
		if (!PyArg_ParseTuple(args, "O&O&:Home", cvtObject, &obj1, cvtObject, &obj2))
		{
			return NULL;
		}
		self->m_pIFace->Home(obj1,obj2);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IShockWeaponSrv>::DestroyMelee(ScriptServiceObject<IShockWeaponSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:DestroyMelee", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->DestroyMelee(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}


} // namespace PyNumbra

#endif
