"""GenericPotion.py

    Demonstrates efficient use of built-in objects and persistence.

    This file is part of PyNumbra
    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from PyNumbra import *
from ScriptLib import *

class GenericPotion:
    
    def GetEffect(self):
        effectName = GetObjectParam(self.m_iObjId,'potion_effect')
        if not effectName or effectName == "":
            return None
        return Object(effectName)
    
    def GetRecipient(self):
        destName = GetObjectParam(self.m_iObjId,'potion_dest')
        if destName:
            return Object(destName)
        else:
            linksrv = ScriptServices.LinkService()
            for l in linksrv.GetAll(LinkKindNamed('ScriptParams'),0,self.m_iObjId):
                if l.GetData() == 'Potion':
                    return l.From()
        return None
    
    def OnStartPotion(self,msg):
        if not self.m_bActive:
            mp = self.GetEffect()
            if mp:
                dest = self.GetRecipient()
                if dest:
                    print "Adding effect %s to %s" % (mp,dest)
                    ScriptServices.ObjectService().AddMetaProperty(dest,mp)
            self.m_bActive = True
        return True
    
    def OnDestroy(self,msg):
        if self.m_bActive:
            mp = self.GetEffect()
            if mp:
                dest = self.GetRecipient()
                if dest:
                    print "Removing effect %s from %s" % (mp,dest)
                    ScriptServices.ObjectService().RemoveMetaProperty(dest,mp)
            self.m_bActive = False
        return True
    
    __Messages__ = { 'StartPotion':OnStartPotion, 'Destroy':OnDestroy }

    def __init__(self,objId):
        self.m_iObjId = objId
        self.m_bActive = False
    
    def __getstate__(self):
        return (self.m_iObjId,self.m_bActive)
    
    def __setstate__(self,persistence):
        (self.m_iObjId,self.m_bActive) = persistence
    

if __name__ == '__main__':
    pass
